<?php



class Transliterator {
	protected $_dataFile;
	protected $_fp;
	protected $_dataPos;

	public function __construct($dataFile) {
		$this->_dataFile = $dataFile;
	}

	protected function _init() {
		if (null === $this->_fp) {
			$this->_fp = fopen($this->_dataFile, 'r');
			$this->_dataPos = current(unpack('N', fread($this->_fp, 4)));
		}
	}

	public function transliterate($string, $toLower=true, $maxLen=30) {
		$transString = '';
		$len = mb_strlen($string, 'UTF-8');
		for ($i=0; $i<$len; $i++) {
			$char = mb_substr($string, $i, 1, 'UTF-8');
			$transChar = $this->transliterateChar($char);
			if (null === $transChar) {
				$transChar = $char;
			}
			$transString .= $transChar;
		}



		$transString =	preg_replace('#^-|-$#', '',
						preg_replace('#-{2,}#', '-',
						preg_replace('#[^\w]#', '-', $transString)));

		if ($toLower) {
			$transString = strtolower($transString);
		}

		if (strlen($transString)>$maxLen) {
			if (preg_match('#^.{0,'.$maxLen.'}(-|$)#', $transString, $matches)) {
				$transString = $matches[0];
			} else {
				$transString = substr($transString, 0, $maxLen);
			}
			$transString = rtrim($transString, '-');
		}

		return $transString;
	}

	public function transliterateChar($char) {
		$this->_init();
		$translitedChar = null;

		$start = 0;
		$end = $this->_dataPos/8;

		while ($end>=$start) {
			$center = floor(($start+$end)/2);

			fseek($this->_fp, 4+$center*8);
			$readChar = rtrim(fread($this->_fp, 4), "\0");

			if (rtrim($readChar, "\0")==$char) {
				$pos = current(unpack('N', fread($this->_fp, 4)));
				fseek($this->_fp, 4+$this->_dataPos+$pos);
				$len = current(unpack('C', fread($this->_fp, 1)));
				$translitedChar = fread($this->_fp, $len);
				break;
			} else if ($char>$readChar) {
				$start = $center+1;
			} else {
				$end = $center-1;
			}
		}

		return $translitedChar;
	}

}


class Transliterate_Controller extends Controller {
	protected $_transliterator;

	public static $allowed_actions = array (
		'Convert'
	);

	public function getTransliterator() {
		if (null === $this->_transliterator) {
			$this->_transliterator = new Transliterator(dirname(__FILE__).'/i18n.dat');
		}
		return $this->_transliterator;
	}

	function Convert($request) {
		$text = $request->postVar('text');
		$convertedText = $this->getTransliterator()->transliterate($text);

		return $this->renderWith('Transliterate', array(
			'convertedText' => json_encode(array(
				'convertedText'=>$convertedText
			))
		));
		
	}
}